# Introduction

This document describes:

- How to install nexus-server for development: useful for developping or contributing to nexus as well as easily testing it
- How to install nexus-server for production
- How to build and run various nexus clients

# Installing nexus-server for development

## Building and installing nexus-server (dev)

First, feel free to modify the server configuration by editing the following file:

- `config/server/nexus.conf`

Next, the `PREFIX` environment variable must be set to the root directory where nexus-server will be installed. Within that directory, a `nexus-server` directory will be created.

In the example below, we set `PREFIX` to the home directory of the current user. This means nexus-server will be installed into `$HOME/nexus-server`.

Run the following commands to build and install nexus-server:
```
make build_srv
make install_dev_srv PREFIX=$HOME
```

## Running nexus-server (dev)

```
make run_srv PREFIX=$HOME
```

## Updating nexus-server binaries (dev)

Stop nexus-server by pressing CTRL-C in the terminal it was started, then run:
```
make build_srv update_srv PREFIX=$HOME
```

## Uninstalling nexus-server (dev)

```
make uninstall_prod_srv PREFIX=$HOME
```

# Installing nexus-server for production

## Building and installing nexus-server (prod)

Start by modifying the following two files which will be used to create the certificates:

1. Modify `config/certs/ca.info` to your liking
1. Modify `config/certs/nexus-server.info` to your liking

Next, feel free to modify the server configuration by editing the following file:

- `config/server/nexus.conf`

Then, the `PREFIX` environment variable must be set to the root directory where nexus-server will be installed. Within that directory, a `nexus-server` directory will be created.

In the example below, we set `PREFIX` to `/usr/local`. This means nexus-server will be installed into `/usr/local/nexus-server`.

Run the following commands to build and install nexus-server:
```
make build_srv
sudo -E make install_prod_srv PREFIX=/usr/local
```

## Running nexus-server (prod)

```
sudo systemctl start nexus-server
```

## Updating nexus-server binaries (prod)

```
make build_srv
sudo systemctl stop nexus-server
sudo -E make update_srv PREFIX=/usr/local
sudo systemctl start nexus-server
```

## Uninstalling nexus-server (prod)

```
sudo systemctl stop nexus-server
sudo -E make uninstall_prod_srv PREFIX=/usr/local
```

# Building and running various nexus clients

## Building nexush for development (dev)

If nexus-server was installed for a development environment, then run the following (to match `PREFIX` that was defined previously for dev):

```
make build_nexush PREFIX=$HOME
```

## Building nexush for production (prod)

However, if nexus-server was installed for a production environment (i.e. as root/sudo), then run the following instead (to match `PREFIX` that was defined previously for prod)::

```
sudo -E make build_nexush PREFIX=/usr/local
```

## Other client targets

There are similar targets for other clients: "build_nexus-cli" and "build_nexus-exam"

## Running nexush

```
make run_nexush EMAIL=admin@nexus.org
```

## Running nexus-exam

For now, the only supported combination of OS and architecture for nexus-exam is Linux amd64. Furthermore, it cannot be built statically (on Ubuntu 20.04 and 22.04 at least) as some static libraries are missing.

```
make run_nexus-exam PREFIX=$HOME
```

# Running validation tests

A variety of tests can be performed in the hope to locate potentiel bugs in the code.
These tests expect a template that includes "xubuntu" in its name, otherwise it will spill out errors.
Furthermore, make sure the login you specify below has access to this template and has most capabilities enabled. 

To run the tests, make sure the nexus-server for development is running, then execute:
```
make tests LOGIN=tests@nexus.org
```
