## REST API

### Login and version management

| Route            | Description                 | Method | Parameters |
|---               |---                          |---     |---         |
| `/login`         | login (return access token) | POST   | email,pwd  |
| `/token/refresh` | obtain a new access token   | GET    |            |
| `/version`       | obtain version number       | GET    |            |

### User management

| Route                 | Description            | Method | Parameters                        | Req. user cap.   |
|---                    |---                     |---     |---                                |---               |
| `/users`              | create a user          | POST   | email,firstname,lastname,pwd,caps | `USER_CREATE`    |
| `/users/{email}`      | delete a user          | DELETE |                                   | `USER_DESTROY`   |
| `/users/{email}/caps` | set caps for a user    | PUT    | caps                              | `USER_SET_CAPS`  |
| `/users/pwd`          | set current user's pwd | PUT    | pwd                               |                  |
| `/users`              | list users             | GET    |                                   | `USER_LIST`      |
| `/users/{email}`      | list a user            | GET    |                                   | `USER_LIST`      |
| `/users/whoami`       | list current user      | GET    |                                   |                  |
     
- Open question: shall we forbid the deletion of a user if they still own templates or VMs?

### VM management

| Route              | Description                                           | Method | Parameters | Req. user cap.   | Op. | Req. VM access cap. |
|---                 |---                                                    |---     |---         |---               |---  |---                  |
| `/vms`             | returns VMs that can be listed                        | GET    |            | `VM_LIST_ANY`    | OR  | `VM_LIST`           |
| `/vms/{id}`        | returns the specified VM                              | GET    |            | `VM_LIST_ANY`    | OR  | `VM_LIST`           |
| `/vms/start`       | returns VMs that can be started                       | GET    |            | `VM_START_ANY`   | OR  | `VM_START`          |
| `/vms/attach`      | returns VM creds info for VMs that can be attached to | GET    |            | `VM_ATTACH_ANY`  | OR  | `VM_ATTACH`         |
| `/vms/{id}/attach` | returns VM creds info for the specified VM            | GET    |            | `VM_ATTACH_ANY`  | OR  | `VM_ATTACH`         |
| `/vms/stop`        | returns VMs that can be killed/shutdown               | GET    |            | `VM_STOP_ANY`    | OR  | `VM_STOP`           |
| `/vms/reboot`      | returns VMs that can be rebooted                      | GET    |            | `VM_REBOOT_ANY`  | OR  | `VM_REBOOT`         |
| `/vms/edit`        | returns VMs that can be edited                        | GET    |            | `VM_EDIT_ANY`    | OR  | `VM_EDIT`           |
| `/vms/editaccess`  | returns VMs that can have their access changed        | GET    |            | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     |
| `/vms/del`         | returns VMs that can be deleted                       | GET    |            | `VM_DESTROY_ANY` | OR  | `VM_DESTROY`        |
| `/vms/exportdir`   | returns VMs that can have a dir downloaded            | GET    |            | `VM_READFS_ANY`  | OR  | `VM_READFS`         |
| `/vms/importfiles` | returns VMs allowing files upload                     | GET    |            | `VM_WRITEFS_ANY` | OR  | `VM_WRITEFS`        |

| Route                      | Description                  | Method | Parameters                     | Req. user cap.   | Op. | Req. VM access cap. |
|---                         |---                           |---     |---                             |---               |---  |---                  |
| `/vms`                     | create a VM                  | POST   | name,cpus,ram,nic,usb,template | `VM_CREATE`      |     |                     |
| `/vms/{id}`                | delete a VM                  | DELETE |                                | `VM_DESTROY_ANY` | OR  | `VM_DESTROY`        |
| `/vms/{id}`                | edit a VM                    | PUT    | name,cpus,ram,nic,usb          | `VM_EDIT_ANY`    | OR  | `VM_EDIT`           |
| `/vms/{id}/start`          | start a VM                   | PUT    |                                | `VM_START_ANY`   | OR  | `VM_START`          |
| `/vms/{id}/startwithcreds` | start a VM with credentials  | PUT    | port,pwd                       | `VM_START_ANY`   | OR  | `VM_START`          |
| `/vms/{id}/stop`           | kill a VM                    | PUT    |                                | `VM_STOP_ANY`    | OR  | `VM_STOP`           |
| `/vms/{id}/reboot`         | reboot a VM                  | PUT    |                                | `VM_REBOOT_ANY`  | OR  | `VM_REBOOT`         |
| `/vms/{id}/shutdown`       | gracefully shutdown a VM     | PUT    |                                | `VM_STOP_ANY`    | OR  | `VM_STOP`           |
| `/vms/{id}/access/{email}` | set VM access for a user     | PUT    | caps                           | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     |
| `/vms/{id}/access/{email}` | del VM access for a user     | DELETE |                                | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     |
| `/vms/{id}/exportdir`      | download a VM's dir          | GET    | dir                            | `VM_READFS_ANY`  | OR  | `VM_READFS`         |
| `/vms/{id}/importfiles`    | upload files into a VM's dir | POST   | files (.tar archive),dir       | `VM_WRITEFS_ANY` | OR  | `VM_WRITEFS`        |

### Template management

| Route                  | Description                          | Method | Parameters       | Req. user cap.                     |
|---                     |---                                   |---     |---               |---                                 |
| `/templates`           | returns templates that can be listed | GET    |                  | `TPL_LIST_ANY` OR `TPL_LIST`       |
| `/templates/{id}`      | returns a template                   | GET    |                  | `TPL_LIST_ANY` OR `TPL_LIST`       |
| `/templates/vm`        | create a template                    | POST   | vmID,name,access | `TPL_CREATE`                       |
| `/templates/qcow`      | create a template                    | POST   | qcow,name,access | `TPL_CREATE`                       |
| `/templates/{id}`      | edit a template                      | PUT    | name,access      | `TPL_EDIT_ANY` OR `TPL_EDIT`       |
| `/templates/{id}`      | delete a template                    | DELETE |                  | `TPL_DESTROY_ANY` OR `TPL_DESTROY` |
| `/templates/{id}/disk` | download a template's disk           | GET    |                  | `TPL_READFS_ANY` OR `TPL_READFS`   |

Remarks:

- A template can only be deleted if no VM references it!

<!--
### Group management

| `/vms/group/{gid}`       | list all VMs in a group                  | GET    |         | `VM_LIST|VM_LIST_ANY`       |
| `/vms/group/{gid}`       | delete all VMs in a group                | DELETE |         | `VM_DESTROY|VM_DESTROY_ANY` |
| `/vms/group/{gid}/start` | start all VMs in a group                 | PUT    |         | `VM_START|VM_START_ANY`     |
| `/vms/group/{gid}/stop`  | stop all VMs in a group                  | PUT    |         | `VM_STOP|VM_STOP_ANY`       |
| `/vms/group/{gid}/mask`  | mask (to non-owner) all VMs in a group   | PUT    |         | `VM_MASK|VM_MASK_ANY`       |
| `/vms/group/{gid}/unmask`| unmask (to non-owner) all VMs in a group | PUT    |         | `VM_MASK|VM_MASK_ANY`       |
| `/vms/{id}/move/{gid}`   | move a VM into a group                   | PUT    |         | `VM_MOVE|VM_MOVE_ANY`       |

| Route                 | Description             | Method | Parameters | Required capabilities         |
|---                    |---                      |---     |---         |---                            |
| `/groups`             | create a group          | POST   | name       | `GRP_CREATE`                  |
| `/groups/{id}`        | delete a group          | DELETE |            | `GRP_DESTROY|GRP_DESTROY_ANY` |
| `/groups/{id}/rename` | rename a group          | PUT    | new name   | `GRP_RENAME|GRP_RENAME_ANY`   |
| `/groups`             | list groups             | GET    |            | `GRP_LIST|GRP_LIST_ANY`       |
-->
