package cmdVM

import (
    "fmt"
    "nexus-common/vm"
    "nexus-client/exec"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type AttachAsyncSingle struct {
    Name string
}

func (cmd *AttachAsyncSingle)GetName() string {
    return cmd.Name
}

func (cmd *AttachAsyncSingle)GetDesc() []string {
    return []string{
        "Attach to a VM.",
        "If not the VM's owner: requires VM_ATTACH VM access capability or VM_ATTACH_ANY user capability."}
}

func (cmd *AttachAsyncSingle)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: ",cmd.GetName(), " ID")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
}

func (cmd *AttachAsyncSingle)Run(args []string) int {
    if err := exec.CheckRemoteViewer(); err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }

    hostname := g.GetInstance().Hostname
    cert := g.GetInstance().PubCert
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    argc := len(args)
    if argc < 1 {
        cmd.PrintUsage()
        return 1
    }

    vmID := args[0]
    resp, err := client.R().Get(host+"/vms/"+vmID+"/attach")
    if err != nil {
        u.PrintlnErr("Failed retrieving VM credentials for VM \""+vmID+"\": "+err.Error())
        return 1
    } else {
        if resp.IsSuccess() {
            myvm, err := deserializeVMCredentials(resp)
            if err != nil {
                u.PrintlnErr("Failed retrieving server's response: "+err.Error())
                return 1
            }

            go func(v vm.VMCredentialsSerialized) {
                stdoutStderr, err := exec.RunRemoteViewer(hostname, cert, v.Name, v.Port, v.Pwd, false)
                if err != nil {
                    u.PrintlnErr("Failed attaching to VM ", v.ID, ": ", fmt.Sprintf("%s", stdoutStderr))
                }
            } (*myvm)
        } else {
            u.PrintlnErr("Failed retrieving VM credentials for VM \""+vmID+"\": "+resp.Status()+": "+resp.String())
            return 1
        }
    }

    return 0
}
