package cmdVM

import (
    "fmt"
    "sync"
    "nexus-client/exec"
    "nexus-common/vm"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type AttachSync struct {
    Name string
}

func (cmd *AttachSync)GetName() string {
    return cmd.Name
}

func (cmd *AttachSync)GetDesc() []string {
    return []string{
        "Attaches to one or more VMs in order to use their desktop environment.",
        "If not the VM's owner: requires VM_LIST VM access capability or VM_LIST_ANY user capability."}
}

func (cmd *AttachSync)PrintUsage() {
    printRegexUsage(cmd)
    printRegexUsageDetails()
}

func (cmd *AttachSync)Run(args []string) int {
    if err := exec.CheckRemoteViewer(); err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }

    hostname := g.GetInstance().Hostname
    cert := g.GetInstance().PubCert

    argc := len(args)
    if argc < 1 {
        cmd.PrintUsage()
        return 1
    }

    vms, err := getFilteredVMCredentials("/vms/attach", args)
    if err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }

    if len(vms) == 0 {
        u.PrintlnErr("Error: VM(s) not found.")
        return 1
    }

    // Use wait groups to wait until all viewers threads have completed.
    var wg sync.WaitGroup
    wg.Add(len(vms))

    statusCode := 0

    for _, v := range(vms) {
        go func(v vm.VMCredentialsSerialized) {
            stdoutStderr, err := exec.RunRemoteViewer(hostname, cert, v.Name, v.Port, v.Pwd, false)
            if err != nil {
                u.PrintlnErr("Failed attaching to VM ", v.ID, ": ", fmt.Sprintf("%s", stdoutStderr))
                statusCode |= 1
            }
            wg.Done()
        } (v)
    }

    wg.Wait()

    return statusCode
}
