package cmdVersion

import (
    "errors"
    "encoding/json"
    "nexus-client/version"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type Version struct {
    Name string
}

func (cmd *Version)GetName() string {
    return cmd.Name
}

func (cmd *Version)GetDesc() []string {
    return []string{
        "Display nexus server and client's versions."}
}

func (cmd *Version)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.GetName())
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
}

func (cmd *Version)Run(args []string) int {
    if len(args) > 0 {
        cmd.PrintUsage()
        return 1
    }

    serverVersion, err := getServerVersion()
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return 1
    }

    u.Println("client version: "+version.Get().String())
    u.Println("server version: "+serverVersion)
    return 0
}

func getServerVersion() (string, error) {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    resp, err := client.R().Get(host+"/version")
    if err != nil {
        return "", err
    } else {
        if resp.IsSuccess() {
            type Response struct {
                Version string
            }
            var response Response
            err = json.Unmarshal(resp.Body(), &response)
            if err != nil {
                return "", err
            }
            return response.Version, nil
        } else {
            return "", errors.New(resp.Status()+": "+resp.String())
        }
    }
}

// Checks the client version is compatible with the server's API.
func CheckServerCompatibility(appname string) bool {
    // Checks the client version is compatible with the server's API.
    v, err := getServerVersion()
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return false
    }
    clientVersion := version.Get()
    serverVersion, _ := version.FromString(v)
    if !clientVersion.IsCompatible(serverVersion) {
        u.PrintlnErr(appname+" version "+clientVersion.String()+" is not compatible with server version "+serverVersion.String()+".")
        if clientVersion.IsOlder(serverVersion) {
            u.PrintlnErr("Please upgrade "+appname+" to version "+serverVersion.StringWithoutBugfix()+".")
        } else {
            u.PrintlnErr("Please downgrade "+appname+" to version "+serverVersion.StringWithoutBugfix()+".")
        }
        return false
    }

    return true
}