package main

import (
    "os"
    "errors"
    "strconv"
    "nexus-common/utils"
    "nexus-client/exec"
    u "nexus-client/utils"
    g "nexus-client/globals"
    "fyne.io/fyne/v2/app"
    "fyne.io/fyne/v2/theme"
    "fyne.io/fyne/v2/widget"
    "fyne.io/fyne/v2/container"
)

const (
    windowTitle = "Live Exam"
)

func run() int {
    if err := exec.CheckRemoteViewer(); err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }

    server, found := os.LookupEnv(g.ENV_NEXUS_SERVER)
    if !found {
        u.PrintlnErr("Environment variable \""+g.ENV_NEXUS_SERVER+"\" must be set!")
        u.PrintlnErr("It defines the nexus server to connect to.")
        u.PrintlnErr("Example: export "+g.ENV_NEXUS_SERVER+"=192.168.1.42")
        return 1
    }

    pubCert, found := os.LookupEnv(g.ENV_NEXUS_CERT)
    if !found {
        u.PrintlnErr("Environment variable \""+g.ENV_NEXUS_CERT+"\" must be set!")
        u.PrintlnErr("It specifies the path to the public certificate required for encrypted communications (TLS) with the nexus server.")
        u.PrintlnErr("Example: export "+g.ENV_NEXUS_CERT+"=ca-cert.pem")
        return 1
    }

    if !utils.FileExists(pubCert) {
        u.PrintlnErr("Failed reading certificate \""+pubCert+"\"!")
        return 1
    }

    app := app.New()
    app.Settings().SetTheme(theme.LightTheme())

    win := app.NewWindow(windowTitle)

    portEntry := widget.NewEntry()
    portEntry.Validator = func(val string) error {
        port, err := strconv.Atoi(val)
        if err != nil {
            return errors.New("Please enter an integer")
        }
        if port < 1024 || port > 65535 {
            return errors.New("Please enter a value within [1024,65535]")
        }
        return nil
    }

    pwdEntry := widget.NewEntry()
    pwdEntry.Password = true

    label := widget.NewLabel("Enter port and password to connect to your VM:")

    form := &widget.Form{
        Items: []*widget.FormItem {
            {Text: "Port", Widget: portEntry},
            { Text: "Password", Widget: pwdEntry},
        },
        OnSubmit: func() {
            port, _ := strconv.Atoi(portEntry.Text)
            exec.RunRemoteViewer(server, pubCert, windowTitle, port, pwdEntry.Text, true)
        },
        SubmitText: "Connect",
    }

    win.SetContent(container.NewPadded(container.NewVBox(label, form)))
    win.ShowAndRun()
    return 0
}

func main() {
    os.Exit(run())
}
