package main

import (
    "os"
    "path"
    "nexus-server/vms"
    "nexus-server/exec"
    "nexus-server/users"
    "nexus-server/utils"
    "nexus-server/router"
    "nexus-server/logger"
    "nexus-server/config"
    "nexus-server/cleaner"
    "nexus-server/version"
    "github.com/sirupsen/logrus"
)

var log = logger.GetInstance()

func main() {
    log.SetLevel(logrus.InfoLevel)

    var appname = path.Base(os.Args[0])
    log.Info(appname+" version "+version.Get().String())

    // Initialize the RNG' seed to the current time.
    utils.RandInit()

    if err := exec.CheckQemuSystem(); err != nil {
        log.Fatal(err)
    }
    if err := exec.CheckQemuImg(); err != nil {
        log.Fatal(err)
    }
    if err := exec.CheckGuestfish(); err != nil {
        log.Fatal(err)
    }

    err := users.InitUsers()
    if err != nil {
        log.Fatal(err.Error())
    }

    // Templates are required by VMs hence they're loaded first.
    err = vms.InitTemplates()
    if err != nil {
        log.Fatal(err.Error())
    }

    err = vms.InitVMs()
    if err != nil {
        log.Fatal(err.Error())
    }

    cleaner.Start()

    router.New().Start(config.GetInstance().Core.APIDefaultPort)
}
