---
title       : |
              Programmation Système\
              \
              Librairies
solutions   : false
author      :
- "Professeur : Florent Gluck"
date        : \today
papersize   : A4
geometry    : "left=2.5cm,right=2cm,top=1.5cm,bottom=2cm"
colorlinks  : urlcolor
fontsize    : 12pt
lang        : fr-CH
---

## Exercice 1 - Création de la lib stat

Créez le répertoire `ex1` qui contiendra le contenu de cet exercice.

On désire créer une librairie nommée `stat`. Cette librairie doit fournir trois fonctions dont voici les prototypes :

- `double average(double *tab, int size);`
- `double min(double *tab, int size);`
- `double max(double *tab, int size);`

En outre, on veut :

1. Que chaque fonction soit implémentée dans un fichier source séparé.
1. Que la librairie soit fournie sous deux formes : statique et dynamique.
1. Que le nom complet de la librarie porte le numéro de version suivante : majeure 1, mineure 3 et révision 7.
1. Que les fichiers correspondant aux noms `SONAME` et de `lien` (*link*) de la librairie soient aussi créés.
1. Implémenter le script bash `validate.sh` qui valide que le `SONAME` contenu dans la librairie est bien correct. Le script prend un argument en entrée: le SONAME à valider. En sortie, il affiche "VALIDATION PASSED" et retourne la valeur 0 si la validation a réussi. Si la validation a échoué, il affiche "VALIDATION FAILED" et retourne la valeur 1.
1. Un Makefile premettant de créer la librairie, selon la syntaxe suivante :
   - `make` génère les deux versions de la librairie (statique et dynamique).
   - `make dyn` génère la version dynamique de la librairie.
   - `make static` génère la version statique de la librairie.
   - `make clean` efface tous les fichiers générés.
   - `make validate` exécute le scripte `validate.sh` ; assurez vous que cette cible possède les bonnes dépendences.

Pour vous aider dans l'écriture du script, `grep` et `awk` sont des amis inestimables. Exemples :

- `cat file|grep pipo` $\rightarrow$ affiche uniquement les lignes contenant la chaîne de caractères `"pipo"`
- `cat file|awk -F":" '{print $5}'` $\rightarrow$ sépare chaque ligne en champs en utilisant `":"` comme séparateur et affiche le 5ème champs

## Exercice 2 - Utilisation de la lib stat pour le développement

Créez le répertoire `ex2` qui contiendra le contenu de cet exercice.

On désire maintenant utiliser la librairie `stat` fraîchement créée pour implémenter le programme exécutable `testat`.
Celui-ci doit simplement afficher la moyenne, le minimum et le maximum d'une liste de nombres réels passés en arguments sur la ligne de commande. Si l'utilisateur ne spécifie pas d'argument(s), `testat` affiche simplement la syntaxe du programme, à savoir :
\small
```
USAGE: testat val1 val2 ...
Where val1, val2, etc. are floating point numbers
```
\normalsize

Dans votre code C, spécifiez la librairie `stat` à utiliser avec la ligne suivante :
\small
```{.c}
#include <stat.h>
```
\normalsize

On part du principe que l'utilisateur-trice n'est pas mal-intentionné(e) et passe uniquement des arguments valides. Notez que la fonction `atof` vous sera probablement très utile (`man atof`).

Plutôt que de coder en dur le nom du programme `testat` dans votre code, utilisez la fonction `basename` en conjonction avec l'argument 0 de la ligne de commande pour obtenir le nom du programme.

Générez deux versions du programme `testat` : une version liée dynamiquement et une autre version liée statiquement.

Sachant que `testat` utilisera la librairie `stat`, créez le sous-répertoire `lib` dans lequel vous copierez les fichiers de la librairie `stat`. Ce répertoire `lib` doit contenir le nombre de fichiers minimum nécessaires. Pensez à utiliser la commande `cp -d` afin de préserver les liens lors de la copie.

Quels fichiers avez-vous copiés dans le répertoire `lib` ?

\begin{comment}
\begin{lstlisting}
mkdir lib
cp -d ../ex1/stat.h lib
cp -d ../ex1/libstat.so.1.3.7 lib
cp -d ../ex1/libstat.so lib
cp -d ../ex1/libstat.a lib
\end{lstlisting}
\end{comment}

Tout comme pour l'exercice précédent, écrivez un Makefile permettant de générer chaque exécutable, dynamique et statique, en respectant les points suivants : 

- l'exécutable `testat` lié en dynamique doit se trouver dans le sous-répertoire `dyn`
- l'exécutable `testat` lié en statique doit se trouver dans le sous-répertoire `static`
- `make` génère les deux versions des exécutables (statique et dynamique)
- `make build.dyn` génère la version dynamique 
- `make build.static` génère la version statique
- `make clean` efface tous les fichiers générés

Assurez-vous que vos dépendences sont correctes. Par exemple, si le répertoire `dyn` existe mais est vide, `make build.dyn` doit uniquement générer l'exécutable `testat` dans le répertoire `dyn`.

Observez le contenu du répertoire `lib` afin de répondre aux questions suivantes :

1. Quels fichier(s) de la librairie `stat` est/sont nécessaire(s) à la génération du programme `dyn/testat` ?
\vspace{.3cm}
1. Quels fichier(s) de la librairie `stat` est/sont nécessaire(s) à la génération du programme `static/testat` ?
\vspace{.3cm}

\begin{comment}
\begin{lstlisting}
dyn/testat:
    stat.h
    libstat.so
    libstat.so.1.3.7 (car libstat.so pointe dessus)

static/testat:
    stat.h
    libstat.a
\end{lstlisting}
\end{comment}


## Exercice 3 - Utilisation de la lib stat pour l'exécution

Pour cet exercice, vous travaillerez également dans le répertoire `ex2`.

Vous venez de générer deux versions de l'exécutable `testat`, une statique et l'autre dynamique. Il ne reste donc plus qu'à les exécuter !

Commencez par vérifier que :

 - l'exécutable `static/testat` est lié statiquement
 - l'exécutable `dyn/testat` est lié dynamiquement

Comment avez-vous déterminé cela ?

\begin{comment}
\begin{lstlisting}
$ file static/testat 
static/testat: ELF 64-bit LSB executable, x86-64, version 1 (GNU/Linux), statically linked, BuildID[sha1]=3c63956edb77e9ff351093c28be73f26253dd344, for GNU/Linux 3.2.0, not stripped

$ file dyn/testat 
dyn/testat: ELF 64-bit LSB shared object, x86-64, version 1 (SYSV), dynamically linked, interpreter /lib64/ld-linux-x86-64.so.2, BuildID[sha1]=f9868a388953edb7171b74b9a6e73f8a33e96ba6, for GNU/Linux 3.2.0, not stripped
\end{lstlisting}
\end{comment}

Ensuite, comparez les tailles des deux exécutables `testat`.

Quelles sont les tailles respectives de chaque exécutable et pourquoi ont-ils des tailles si différentes ?

\begin{comment}
\begin{lstlisting}
dyn/testat: 17KB
static/testat: 897KB
\end{lstlisting}
\end{comment}

Passez maintenant à l'exécution :

1. Déplacez-vous dans le répertoire `ex2/static/` et exécutez `testat`.
   - Est-ce que celui-ci s'exécute correctement ?

\begin{comment}
\begin{lstlisting}
Oui, il s'exécute correctement: ./static/testat
\end{lstlisting}
\end{comment}

\vspace{.3cm}
1. Déplacez-vous dans le répertoire `ex2/dyn/` et exécutez `testat`.
   - Est-ce que celui-ci s'exécute correctement ?
   - Pourquoi ne peut-il pas être exécuté, contrairement à la version statique ?
   - Que devez-vous faire pour exécuter `ex2/dyn/testat` ?

\begin{comment}
\begin{lstlisting}
Non, il ne fonctionne pas car le chargeur ne sait pas où trouver la librairie libstat.so.1 dont dépend l'exécutable :
$ ./testat 
./testat: error while loading shared libraries: libstat.so.1: cannot open shared object file: No such file or directory

Il faut donc rajouter cette librairie, p.ex. dans le répertoire lib, puis indiquer au chargeur ce nouveau répertoire car ce n'est pas un répertoire système qu'il inspecte par défaut :

$ cp -d ../ex1/libstat.so.1 lib
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:lib
./dyn/testat
\end{lstlisting}
\end{comment}

Maintenant que vous avez réussi à exécuter chaque programme, vous devriez pouvoir facilement répondre à ces questions :

1. Quels fichier(s) de la librairie `stat` est/sont nécessaire(s) à l'exécution du programme `dyn/testat` ?
\vspace{.3cm}
1. Quels fichier(s) de la librairie `stat` est/sont nécessaire(s) à l'exécution du programme `static/testat` ?
\vspace{.3cm}


\begin{comment}
\begin{lstlisting}
dyn/testat:
libstat.so.1
libstat.so.1.3.7 (car libstat.so.1 pointe dessus)

static/testat:
aucun
\end{lstlisting}
\end{comment}

<!--
## Exercice 4 - Comparaisons

On génère la même lib de 2 manières différentes :

1) Toutes les fonctions sont dans le même .o
1) Chaque fonction se trouve dnans un .o séparé

A partir du même code source, on génère 2 exécutables liés **statiquement** :

- Remarquez-vous une différence de taille entre ces 2 exécutables ?
  - Comment expliquez vous les tailles observées ?

Ensuite, à partir du même code source, on génère 2 exécutables liés **dynamiquement** :

- Remarquez-vous une différence de taille entre ces 2 exécutables ?
  - Comment expliquez vous les tailles observées ?
-->