---
author: Florent Gluck - Florent.Gluck@hesge.ch

title: Infrastructure de bureau virtuel (VDI) avec Nexus

date: \vspace{.5cm} \footnotesize \today

pandoc-latex-fontsize:
  - classes: [tiny]
    size: tiny
  - classes: [verysmall]
    size: scriptsize
  - classes: [small]
    size: footnotesize
  - classes: [huge, important]
    size: huge
---

[//]: # ----------------------------------------------------------------
## Modèle d'infrastructure de bureau classique

- Chaque utilisateur-trice possède un ordinateur dans lequel est installé un système d'exploitation (OS) qui exécute ses applications
- Modèle **local** $\rightarrow$ tout s'exécute sur la machine de l'utilisateur-trice

\vspace{.5cm}

\centering
![](images/client_pc.png){ width=30% }

[//]: # ----------------------------------------------------------------
## Modèle d'infrastructure de bureau virtuel (VDI)

::: incremental :::

- Chaque utilisateur-trice possède une machine virtuelle (VM) dans laquelle est installé un OS qui exécute ses applications
- Chaque utilisateur-trice possède un ordinateur très léger $\rightarrow$ utilisé **uniquement** pour afficher et interagir avec le bureau
- Les VM **s'exécutent** sur un serveur **distant** hébergé dans un centre de calculs/données
- Image du bureau transmise via le réseau jusqu'à l'ordinateur de l'utilisateur-trice
- Utilisation **identique** à un ordinateur local
- Modèle distant $\rightarrow$ **tout s'exécute sur le serveur distant**

:::

[//]: # ----------------------------------------------------------------
## Exemple de bureau virtuel (Nexus)

\centering
![](images/VM_xubuntu_desktop.png){ width=95% }

[//]: # ----------------------------------------------------------------
## Architecture VDI

\centering
![](images/vdi_arch.png){ width=90% }

[//]: # ----------------------------------------------------------------
## Nexus

::: incremental :::

Infrastructure de bureau virtuel (VDI) développée à ISC

- Examens *live* sur PC **sans possibilité de fraude**
- Environnements **personnalisés** pour laboratoires/exercices/travaux pratiques
- Utilisable par **tout-e enseignant-e**
- Machines virtuelles et droits d'accès **hautement configurable**
- Accessible depuis **l'extérieur** via VPN
- **Sécurisé** via communications chiffrées

:::

[//]: # ----------------------------------------------------------------
## Architecture Nexus

\centering
![](images/nexus_arch.png){ width=90% }

[//]: # ----------------------------------------------------------------
## Examens sans fraude avec Nexus

\centering
![](images/exam_with_nexus.png){ width=90% }

[//]: # ----------------------------------------------------------------
## Préparation examen avec Nexus

Avant l'examen (typiquement la veille), l'enseignant-e :

::: incremental :::

1. Prépare une VM *template* pour l'examen (énoncé/questions + logiciels nécessaires)
   - VM typiquement configurée sans accès Internet (ou seulement certains sites autorisés)
1. Créé une VM par étudiant-e
1. Démarre les VM des étudiant-e-s
1. Génère les accès uniques à chaque étudiant-e sur papier (1 bande papier/étudiant-e)

:::

[//]: # ----------------------------------------------------------------
## Déroulement examen avec Nexus

:::::: {.columns}
::: {.column width="60%"}

Le jour de l'examen :

::: incremental :::

\small

1. Enseignant-e met à disposition un laptop à chaque étudiant-e\textsuperscript{*}
   - \footnotesize démarre automatiquement sur Nexus-exam via réseau câblé (ou wifi HES-GE) ou clé USB dédiée
1. Enseignant-e fourni à chaque étudiant-e ses accès (papier)
1. Etudiant-e se connecte avec ses accès à sa VM et réalise l'examen
1. Etudiant-e éteint sa VM une fois l'examen terminé

:::

:::
::: {.column width="40%"}

\centering
![](images/nexus-exam.png){ width=100% }

\vspace{.5cm}

\centering
![](images/vmcred2pdf.png){ width=100% }

:::
::::::

\vfill
\textsuperscript{*}\scriptsize variante : possibilité d'utiliser le laptop de l'étudiant-e

[//]: # ----------------------------------------------------------------
## Evaluation examen avec Nexus

Lorsque l'enseignant-e est prêt-e à évaluer les examens :

::: incremental :::

1. Télécharge depuis chaque VM les fichiers produits par les étudiants sur son PC
1. Évalue les fichiers produits
   - peut également utiliser la VM de l'étudiant-e afin de tester ce qu'il s'y trouve
1. Supprime les VM de tous les étudiant-e-s

:::

<!--
[//]: # ----------------------------------------------------------------
## Démonstration de Nexus

![](images/demo_time.png){ height=100% }
-->

[//]: # ----------------------------------------------------------------
## Retour d'expérience

- Nexus utilisé **\textcolor{mygreen}{avec succès}** dans le cadre d'examens *live* dans la filière ISC à HEPIA d'octobre 2022 à Juin 2023

\vspace{.5cm}

\centering
![](images/thumb_up.png){ width=50% }

[//]: # ----------------------------------------------------------------
## Perspectives futures

- Ajouts de nouvelles fonctionalités
  - interface graphique
  - gestion réseaux virtuels
  - VM Windows
  - ...
- Utilisation par les autres HES
- Demande de fonds dans le cadre d'un projet enseignement numérique (stratégie numérique HES-SO)

<!--
[//]: # ----------------------------------------------------------------
## Questions

\centering
![](images/questions.png){ width=80% }
-->